class Deals(object):
    BASE_URL = "crm/v3/objects/deals/"

    def __init__(self, client):
        self._client = client

    def get_deals(self, **params):
        """
        Get all of the deals in an account.  Returns a paginated set of deals.

        The paging cursor token of the last successfully read resource will be returned as the paging.next.after JSON property of a paged response containing more results.
        """
        return self._client._get(self.BASE_URL, params=params)

    def create_deal(self, data):
        """
        For a given account, create a deal. This is particularly useful if you're
        integrating with a CRM or another application that has a similar notion of
        a deal.

        Note that you can create associations between Deals and Contacts and Companies
        much the same way you create an association between Companies and Contacts.
        Associating your Deal with Contacts and Companies is not required (i.e. you
        can create an orphaned deal). You can also pass any custom property value you
        want to the deals API, so long as you've created the property first.

        The dealstage property is required when creating a deal. If the pipeline
        property is not specified, the default pipeline is assumed. However, it is
        recommended to always specify the pipeline, especially on accounts with multiple
        pipelines. You can manage pipelines and dealstages through the CRM Pipelines API.
        """
        data = {"properties": {k: v for k, v in data.items()}}
        return self._client._post(self.BASE_URL, json=data)

    def delete_deal(self, deal_id):
        """
        Deletes the exisiting deal specified by :dealId.
        """
        return self._client._delete(self.BASE_URL + str(deal_id))

    def get_deal(self, deal_id, **params):
        """
        Returns an object representing the deal with the id :dealId associated
        with the specified account.

        Two important fields will be returned to you that identify the appropriate
        associations with the deal: associatedCompanyIds returns the id of the
        company associated with the deal, and associatedVids returns the ids of the
        contacts associated with the deal.

        You can then perform lookups on either the contacts or the company.
        """
        return self._client._get(self.BASE_URL + str(deal_id), params=params)



class Companies(object):
    BASE_URL = "crm/v3/objects/companies/"

    def __init__(self, client):
        self._client = client

    def get_companies(self, **params):
        """
        Used to get all of the companies in a account. Returns a paginated
        list of companies.

        The paging cursor token of the last successfully read resource will
        be returned as the paging.next.after JSON property of a paged response
        containing more results.
        """
        return self._client._get(self.BASE_URL, params=params)

    def create_company(self, data):
        """
        For a given account, create a company. This is particularly useful
        if you're integrating with a CRM or another application that has a
        similar notion of a company.

        This will return an object representing your newly created company.
        """
        data = {"properties": {k: v for k, v in data.items()}}
        return self._client._post(self.BASE_URL, json=data)

    def delete_company(self, company_id):
        """
        Deletes the exisiting company specified by :companyId. Returns JSON
        indicating whether or not the specified company was actually deleted.

        Since companies play a central role in the CRM, it is a best practice
        not to delete a company unless your application has created them.
        """
        return self._client._delete(self.BASE_URL + str(company_id))

    def get_company(self, company_id, **params):
        return self._client._get(self.BASE_URL + str(company_id), params=params)



class ContactLists(object):
    BASE_URL = "contacts/v1"

    def __init__(self, client):
        self._client = client

    def get_contact_lists(self, **params):
        """
        Return the contact lists for a portal.

        By default, you will get up to 20 lists to you at a time.
        You can get more lists in a single request (up to 250) using
        the count parameter. Each response will include an offset,
        which can be used with the offset= parameter in the next
        request to get the next page of lists.

        The listId is the account specific ID for the list.
        This value never changes for a particular list. The internalListId
        is an internal identifier used to track the list criteria revisions,
         and is subject to change when the list is modified.

        See the Contact List Overview for details about the returned data
        """
        endpoint = "/lists"
        return self._client._get(self.BASE_URL + endpoint, params=params)

    def create_contact_list(self, data):
        """
        Create a new list in a given HubSpot account to populate with contacts.

        Creating this list show in the user interface, so beware that users
        will be able to edit and even delete lists that are programatically
        created in HubSpot.
        """
        endpoint = "/lists"
        return self._client._post(self.BASE_URL + endpoint, json=data)

    def delete_contact_list(self, contact_list_id):
        """
        Delete a list in a given HubSpot account, identified by its unique ID.

        Note that lists can be used by users to trigger marketing automation
        campaigns, so a good best practice is to only delete the lists that
        your integration with HubSpot has created.
        """
        endpoint = "/lists/{}".format(contact_list_id)
        return self._client._delete(self.BASE_URL + endpoint)

    def get_recently_added_contacts_in_a_list(self, **params):
        """
        For a given HubID and a given list, return contacts that have been
        recently added to that list, starting with the most recently added
        contact and pageing backwards. A paginated list of contacts will be
        returned to you, with a maximum of 100 contacts per page, as specified
        by the "count" parameter.
        """
        endpoint = "/lists/recently_updated/contacts/recent"
        return self._client._get(self.BASE_URL + endpoint, params=params)

    def get_contact_list(self, contact_list_id, **params):
        """
        For a given portal, return a contact list by its unique ID. This returns
         only the metadata for the list; see this page for getting the contact
         records in the list.

        If you are not storing the list ids inside of your application, you'll
        need to first find the list id by using the get all lists endpoint.

        See the Contact List Overview for details about the returned data
        """
        endpoint = "/lists/{}".format(contact_list_id)
        return self._client._get(self.BASE_URL + endpoint, params=params)

    def add_contact_to_list(self, contact_list_id, data):
        """
        Add contact records that have already been created in the system to
        a contact list. You can add multiple records at once, either by vid
        or by email address. Up to 500 records can be added to a list in a
        single request, including records specified by ID and by email.

        Please note that you cannot manually add contacts to dynamic lists.
        To determine whether a list is dynamic or static, when you get a list,
        you will see a flag called dynamic that equates to true or fals
        """
        endpoint = "/lists/{}/add".format(contact_list_id)
        return self._client._post(self.BASE_URL + endpoint, json=data)



class Webhooks(object):
    BASE_URL = "webhooks/v1"

    def __init__(self, client):
        self._client = client

    def get_settings(self):
        endpoint = "/{}/settings".format(self._client.app_id)
        return self._client._get(self.BASE_URL + endpoint, params={"hapikey": self._client.hapikey})

    def update_settings(self, data):
        endpoint = "/{}/settings".format(self._client.app_id)
        return self._client._put(self.BASE_URL + endpoint, json=data, params={"hapikey": self._client.hapikey})

    def get_subscriptions(self):
        endpoint = "/{}/subscriptions".format(self._client.app_id)
        return self._client._get(self.BASE_URL + endpoint, params={"hapikey": self._client.hapikey})

    def create_subscription(self, data):
        endpoint = "/{}/subscriptions".format(self._client.app_id)
        return self._client._post(self.BASE_URL + endpoint, json=data, params={"hapikey": self._client.hapikey})

    def update_subscription(self, subscription_id, data):
        endpoint = "/{}/subscriptions/{}".format(self._client.app_id, subscription_id)
        return self._client._post(self.BASE_URL + endpoint, json=data, params={"hapikey": self._client.hapikey})

    def delete_subscription(self, subscription_id):
        endpoint = "/{}/subscriptions/{}".format(self._client.app_id, subscription_id)
        return self._client._delete(self.BASE_URL + endpoint, params={"hapikey": self._client.hapikey})



from enum import Enum


class ErrorEnum(Enum):
    Unauthorized = 401
    Forbidden = 403
    TooManyRequests = 429
    Timeout1 = 502
    Timeout2 = 504
    NotFound = 404
    MethodNotAllowed = 405



class Workflows(object):
    BASE_URL = "automation/v3"

    def __init__(self, client):
        self._client = client

    def get_workflows(self):
        """
        Get a list of workflows for a HubSpot portal. This endpoint is
        particularly useful if you are looking for a particular workflow
        and need to look it up by an Id. The endpoint only returns metadata
        for each workflow.
        """
        endpoint = "/workflows"
        return self._client._get(self.BASE_URL + endpoint)

    def get_workflow(self, workflow_id, **params):
        """
        Returns metadata for a workflow.

        This endpoint will not return the contacts that are enrolled in the
        workflow. Workflow IDs can be found by using the get all workflows endpoint.
        """
        endpoint = "/workflows/{}".format(workflow_id)
        return self._client._get(self.BASE_URL + endpoint, params=params)

    def create_workflow(self, data):
        """
        Create a new workflow in a portal. Workflows are an essential
        component of the HubSpot marketing product, allowing marketing
        automation that relies on triggering relevant and timely actions,
        based on a user's context.
        """
        endpoint = "/workflows"
        return self._client._post(self.BASE_URL + endpoint, json=data)

    def delete_workflow(self, workflow_id):
        """
        Delete a workflow. Because deleting a workflow could be very
        disruptive to a user - you might be deleting a marketing automation
        workflow - a good best practice is to only delete workflows that your
        integration created.
        """
        endpoint = "/workflows/{}".format(workflow_id)
        return self._client._delete(self.BASE_URL + endpoint)

    def enroll_a_contact_into_workflow(self, workflow_id, contact_email):
        """
        Add a contact to a workflow.

        You will want to use this in the event that a contact meets a set
        of criteria that makes enrolling them in a workflow relevant to them,
        for example downloading a whitepaper and then getting enrolled in a
        workflow that sends emails based on the content of that whitepaper.
        """
        endpoint = "/workflows/{}/enrollments/contacts/{}".format(workflow_id, contact_email)
        return self._client._post("automation/v2" + endpoint)

    def unenroll_a_contact_into_workflow(self, workflow_id, contact_email):
        """
        Remove a contact from a workflow.

        If a contact is removed from a workflow, all future events that
        they are scheduled for will not take place. One application of this
        endpoint could be removing a contact from a marketing automation workflow
        if they are closed as a customer in your CRM.
        """
        endpoint = "/workflows/{}/enrollments/contacts/{}".format(workflow_id, contact_email)
        return self._client._delete("automation/v2" + endpoint)



from hubspot import exceptions


class Fields(object):
    BASE_URL = "crm/v3/properties/{}"

    def __init__(self, client):
        self._client = client

    def get_fields(self, module):
        if module in ["deals", "companies", "contacts"]:
            return self._client._get(self.BASE_URL.format(module))

        raise exceptions.ModuleNotFoundError('Your must provide "deals", "companies", "contacts" such a module')



class Contacts(object):
    BASE_URL = "crm/v3/objects/contacts/"

    def __init__(self, client):
        self._client = client

    def get_contacts(self, **params):
        """
        For a given account, return all contacts that have been created
        in the account.

        A paginated list of contacts will be returned to you, with a
        maximum of 10 contacts per page by default.
        """
        return self._client._get(self.BASE_URL, params=params)

    def create_contact(self, data):
        """
        Create a new contact in HubSpot with a simple HTTP POST to the Contacts API.
        The contact will be created instantly inside of HubSpot, and will be assigned
        a unique ID (vid) that can be used to look up the contact inside of HubSpot later.-
        """
        data = {"properties": {k: v for k, v in data.items()}}
        return self._client._post(self.BASE_URL, json=data)

    def delete_contact(self, contact_id):
        """
        Delete an existing contact from a particular HubSpot portal.

        If a contact with the same email address interacts with the portal again
        (via a form submission for example) the contact will be added back into
        the user interface.
        """
        return self._client._delete(self.BASE_URL)

    def get_contact(self, contact_id, **params):
        """
        For a given account, return information about a single contact by its ID.
        The contact's unique ID is stored in a field called 'vid' which stands for
        'visitor ID'.

        This method will also return you much of the HubSpot lead "intelligence"
        that you may be accustomed to getting from the leads API, as properties
        in this new API. More of this intelligence will be available as time passes,
        but this call is where you can expect to find it.
        """
        return self._client._get(self.BASE_URL + str(contact_id), params=params)



class BaseError(Exception):
    pass


class UnauthorizedError(BaseError):
    pass


class ForbiddenError(BaseError):
    pass


class TooManyRequestsError(BaseError):
    pass


class TimeoutError(BaseError):
    pass


class UnexpectedError(BaseError):
    pass


class NotFoundError(BaseError):
    pass


class MethodNotAllowedError(BaseError):
    pass


class ModuleNotFoundError(BaseError):
    pass



class Integrations(object):
    BASE_URL = "integrations/v1"

    def __init__(self, client):
        self._client = client

    def get_account_details(self, **params):
        """
        Get the details for a HubSpot account, based on the provided API key or
        access token.

        Currently, this will include the Portal ID (often called the Hub ID) of
        the account, the time zone setting of the account, and the selected currency
        of the account.
        """
        endpoint = "/me"
        return self._client._get(self.BASE_URL + endpoint, params=params)

    def get_daily_api_usage(self, **params):
        """
        Check to see how many API calls that have been made for a portal for
        the current day, as well as the time that the limit will reset.
        The current day is measured from midnight to midnight based on the
        time zone setting of the portal.

        See the API Usage Guidelines for more details about HubSpot's API
        limits, and this page for details about working with those limits.

        The data returned returned by this endpoint will be cached for 5 minutes.
        Check the fetchStatus and collectedAt fields in the response to determine
        if the response was from cache.
        """
        endpoint = "/limit/daily"
        return self._client._get(self.BASE_URL + endpoint, params=params)



import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="hubspot-python",
    version="1.0.2",
    description="HubSpot API written in python",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Lelia Rubiano",
    author_email="lrubiano5@gmail.com",
    url="https://github.com/GearPlug/hubspot-python.git",
    packages=["hubspot"],
    install_requires=[
        "requests",
    ],
    keywords=["hubspot", "crm"],
    zip_safe=False,
    license="MIT",
)



import requests

from hubspot import exceptions
from hubspot.enums import ErrorEnum
from hubspot.companies import Companies
from hubspot.contact_lists import ContactLists
from hubspot.contacts import Contacts
from hubspot.deals import Deals
from hubspot.fields import Fields
from hubspot.integrations import Integrations
from hubspot.webhooks import Webhooks
from hubspot.workflows import Workflows
from urllib.parse import urlencode, urlparse


class Client(object):
    BASE_URL = "https://api.hubapi.com/"

    def __init__(self, app_id, hapikey, client_id, client_secret):
        self.app_id = app_id
        self.hapikey = hapikey
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = None

        self.companies = Companies(self)
        self.contact_lists = ContactLists(self)
        self.contacts = Contacts(self)
        self.deals = Deals(self)
        self.fields = Fields(self)
        self.integrations = Integrations(self)
        self.webhooks = Webhooks(self)
        self.workflows = Workflows(self)

    def authorization_url(self, redirect_uri, scope):
        if not isinstance(scope, list):
            raise Exception("scope must be a list.")
        params = {"client_id": self.client_id, "redirect_uri": redirect_uri, "scope": " ".join(scope)}
        url = "https://app.hubspot.com/oauth/authorize?" + urlencode(params)
        return url

    def exchange_code(self, redirect_uri, code):
        data = {
            "grant_type": "authorization_code",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": redirect_uri,
            "code": code,
        }
        return self._post("oauth/v1/token", data=data)

    def refresh_token(self, refresh_token):
        data = {
            "grant_type": "refresh_token",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "refresh_token": refresh_token,
        }
        return self._post("oauth/v1/token", data=data)

    def set_access_token(self, access_token):
        self.access_token = access_token

    def _get(self, endpoint, **kwargs):
        return self._request("GET", endpoint, **kwargs)

    def _post(self, endpoint, **kwargs):
        return self._request("POST", endpoint, **kwargs)

    def _put(self, endpoint, **kwargs):
        return self._request("PUT", endpoint, **kwargs)

    def _delete(self, endpoint, **kwargs):
        return self._request("DELETE", endpoint, **kwargs)

    def _request(self, method, endpoint, headers=None, params=None, **kwargs):
        _headers = {"Authorization": "Bearer {}".format(self.access_token)}
        if params and "hapikey" in params:
            del _headers["Authorization"]
        if headers:
            _headers.update(headers)
        return self._parse(
            requests.request(method, self.BASE_URL + endpoint, headers=_headers, params=params, **kwargs)
        )

    def _parse(self, response):
        print(response.request.url)
        if "Content-Type" in response.headers and "application/json" in response.headers["Content-Type"]:
            r = response.json()
        else:
            r = response

        if not response.ok:
            code = response.status_code
            message = r.get("message", None)
            try:
                error_enum = ErrorEnum(code)
            except Exception:
                raise exceptions.UnexpectedError("{}".format(message))

            if error_enum == ErrorEnum.Unauthorized:
                raise exceptions.UnauthorizedError(message)
            elif error_enum == ErrorEnum.Forbidden:
                raise exceptions.ForbiddenError(message)
            elif error_enum == ErrorEnum.TooManyRequests:
                raise exceptions.TooManyRequestsError(message)
            elif error_enum == ErrorEnum.Timeout1:
                raise exceptions.TimeoutError(message)
            elif error_enum == ErrorEnum.Timeout2:
                raise exceptions.TimeoutError(message)
            elif error_enum == ErrorEnum.NotFound:
                raise exceptions.NotFoundError(message)
            elif error_enum == ErrorEnum.MethodNotAllowed:
                raise exceptions.MethodNotAllowedError(message)
            else:
                raise exceptions.BaseError("{}".format(message))

        return r
